#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "baseproxy4xi.py">
#<abstract4xi abstract = "provides BaseProxy4Xi class"/>
#<refer4xi reference = "managers.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'BaseProxy4Xi' ]
#"
#/> 

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except: 
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import weakref
    import array
    import queue
    from traceback import format_exc         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.managers import BaseProxy
except:
    print("conditionproxy4xi.py *** failed in from multiprocessing.managers import BaseProxy ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from constant_family4xi import ConstantFamily4Xi
from constant_managers4xi import get_return_manager4xi 
from constant_managers4xi import get_proxy_manager4xi
from constant_managers4xi import get_getvalue_manager4xi
from constant_managers4xi import get_exitpriority_manager4xi
from constant_managers4xi import get_shutdown_manager4xi
from constant_managers4xi import get_started_manager4xi
from convert_to_error4xi import convert_to_error4xi
from current_process4xi import get_authkey_current_process4xi
from current_process4xi import get_name_current_process4xi
from dispatch4xi import dispatch4xi
from finalize4xi import Finalize4Xi
from forkawarelocal4xi import ForkAwareLocal4Xi
from forkawarethreadlock4xi import ForkAwareThreadLock4Xi
from listener_client4xi import listener_client4xi
from popen4xi import Popen4Xi
from processlocalset4xi import ProcessLocalSet4Xi
from reason_object4xi import reason_object4xi
from rebuildproxy4xi import RebuildProxy4Xi
from register_after_fork4xi import register_after_fork4xi
from sense_object4xi import sense_object4xi
from state4xi import State4Xi
from print4xi import print4xi
from sleep4xi import omega2sleep4xi
from timeout4xi import omega2timeout4xi
from time4xi import time4xi
#"
#/>

# =================================================================================================
#<class4xi class = "BaseProxy" base = "object" child = "IteratorProxy4Xi, AcquirerProxy4Xi, EventProxy4Xi, NamespaceProxy4Xi, ValueProxy4Xi, ListProxy4Xi" 
# synopsis = "Provides a base for proxies of shared objects"
# usage = "
#    managers.py
#        MakeProxyType(name, exposed, _cache={}): 
#        ProxyType = type(name, (BaseProxy,), dic)
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class BaseProxy4Xi(object):
    '''
    BaseProxy4Xi: a base for proxies of shared objects
    '''
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>
                                 
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _address_to_local = {}
    _mutex = ForkAwareThreadLock4Xi()
    _cf2xi = ConstantFamily4Xi()
    #"
    #/>
           
   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-BaseProxy4Xi" usage = "initializes">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, token, serializer, manager=None, authkey=os.urandom(32), exposed=None, incref=True):
        '''
        initializes BaseProxy4Xi 
        '''
  
        # -----------------------------------------------------------------------------------------    
        # Step 0/5
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BaseProxy4Xi'
        method2print = '__init__'
        
        
        message2print = "{(Step 0/5a) started" + "; init-of-BaseProxy4Xi-started: " + time4xi() + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        message2print = "{(Step 0/5b) passed; token-passed-to-init-of-BaseProxy4Xi: " + str(token) + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
           
        message2print = "{(Step 0/5c) passed; serializer-passed-to-init-of-BaseProxy4Xi: " + str(serializer) + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
          
        message2print = "{(Step 0/5d) passed; manager-passed-to-init-of-BaseProxy4Xi: " + str(manager) + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                  
        message2print = "{(Step 0/5e) passed; authkey-passed-to-init-of-BaseProxy4Xi: " + str(authkey) + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                 
        message2print = "{(Step 0/5f) passed; exposed-passed-to-init-of-BaseProxy4Xi: " + str(exposed) + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                   
        message2print = "{(Step 0/5g) passed; incref-passed-to-init-of-BaseProxy4Xi: " + str(incref) + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        message2print = "{(Step 0/5h) borning" + "; baseproxy2xi-of-BaseProxy4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 0/5i) inheriting" + "; object-inheriting-by-baseproxy2xi-of-BaseProxy4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 0/5j) sharing" + "; address-to-local-sharing-by-baseproxy2xi-of-BaseProxy4Xi: " + str(BaseProxy4Xi._address_to_local)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                  
        message2print = "{(Step 0/5k) sharing" + "; mutex-sharing-by-baseproxy2xi-of-BaseProxy4Xi: " + str(BaseProxy4Xi._mutex)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
            
        
        # -----------------------------------------------------------------------------------------    
        # Step 1/5
        # ----------------------------------------------------------------------------------------- 
        BaseProxy4Xi._mutex.acquire()
        
        message2print = "{(Step 1/5 ) performed" + "; BaseProxy4Xi-mutex-acquire-performed-by-init-of-BaseProxy4Xi: " + str(BaseProxy4Xi._mutex.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
        # -----------------------------------------------------------------------------------------    
        # Step 2/5
        # -----------------------------------------------------------------------------------------        

        try:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            message2print = "{passing" + "; token-address-passing-to-BaseProxy4Xi-by-init-of-BaseProxy4Xi: " + str(token.address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            

            tls_idset = BaseProxy4Xi._address_to_local.get(token.address, None)
            
            message2print = "{set" + "; tls-idset-set-by-init-of-BaseProxy4Xi to: " + str(tls_idset)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
            if tls_idset is None:
                
                tls_idset = ForkAwareLocal4Xi(), ProcessLocalSet4Xi()
        
                message2print = "{set" + "; tls-idset-set-by-init-of-BaseProxy4Xi to: " + str(tls_idset)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                

                
                BaseProxy4Xi._address_to_local[token.address] = tls_idset
                

                message2print = "{(Step 2/5a) set" + "; BaseProxy-token-address-set-by-init-of-BaseProxy4Xi to: " + str(BaseProxy4Xi._address_to_local[token.address])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
        finally:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
        
            BaseProxy4Xi._mutex.release()
            
     
            message2print = "{(Step 2/5b) performed" + "; BaseProxy4Xi-mutex-release-performed-by-init-of-BaseProxy4Xi: " + str(BaseProxy4Xi._mutex.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                
            
            
        # -----------------------------------------------------------------------------------------    
        # Step 3/5
        # -----------------------------------------------------------------------------------------
        # self._tls is used to record the connection used by this
        # thread to communicate with the manager at token.address

        self._tls = tls_idset[0]
                
        message2print = "{(Step 3/5a) set" + "; tls-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._tls)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
        message2print = "{(Step 3/5b) commenting" + "; tls-commenting-by-init-of-BaseProxy4Xi: tls is used to record the connection used by the thread to communicate with the manager at token.address " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
                
        # -----------------------------------------------------------------------------------------    
        # Step 4/5
        # -----------------------------------------------------------------------------------------
        # self._idset is used to record the identities of all shared
        # objects for which the current process owns references and
        # which are in the manager at token.address
        
        message2print = "{(Step 4/5a) commented; baseproxy2xi-idset-commented: " + "idset is used to record the identities of all shared objects for which the current process owns references and which are in the manager" + "; time: " + time4xi() +"}"
        print4xi(class2print, method2print, message2print)   
        
 
           
        self._idset = tls_idset[1]
        message2print = "{(Step 4/5a) commented; idset-commented-by-init-of-BaseProxy4Xi: idset is used to record the identities of all shared; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 4/5b) commented; idset-commented-by-init-of-BaseProxy4Xi: objects for which the current process owns references; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        message2print = "{(Step 4/5c) commented; idset-by-init-of-BaseProxy4Xi to-commented: " + " which are in the manager at token.address; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 4/5d) set; idset-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._idset)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
           
        self._token = token
                
        message2print = "{(Step 4/5e) set; token-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._token)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
             
        self._id = self._token.id
      
        message2print = "{(Step 4/5f) set; id-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._id)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
             
        self._manager = manager
        message2print = "{(Step 4/5g) set; manager-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._manager)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
             
        self._serializer = serializer
        message2print = "{(Step 4/5h) set; serializer-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._serializer)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
             
        self._Client = listener_client4xi[serializer][1]
              
        message2print = "{(Step 4/5i) set; Client-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._Client)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
     
        if authkey is not None:
            self._authkey = authkey 
        elif self._manager is not None:
            self._authkey = self._manager._authkey
        else:
            self._authkey = get_authkey_current_process4xi()
        
        message2print = "{(Step 4/5j) set; authkey-of-baseproxy2xi-set-by-init-of-BaseProxy4Xi to: " + str(self._authkey)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
           
        if incref:
            self._incref()
            message2print = "{(Step 4/5k) performed" + "; incref-of-baseproxy2xi-performed-by-init-of-BaseProxy4Xi: " + str(self._incref) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                        

                      
              
        # -----------------------------------------------------------------------------------------    
        # Step 5/5
        # -----------------------------------------------------------------------------------------
        register_after_fork4xi(self, BaseProxy4Xi._after_fork)
     
        message2print = "{(Step 5/5a) performed" + "; register-after-fork4xi-performed-by-init-of-BaseProxy4Xi: "  + str(register_after_fork4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        
        message2print = "{(Step 0/5b) inherited" + "; object-inherited-by-baseproxy2xi-of-BaseProxy4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
             
        message2print = "{(Step 5/5c) activated" + "; baseproxy2xi-of-BaseProxy4Xi-activated: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)

        message2print = "{(Step 5/5d) shared" + "; address-to-local-shared-by-baseproxy2xi-of-BaseProxy4Xi: " + str(BaseProxy4Xi._address_to_local)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                  
        message2print = "{(Step 5/5e) shared" + "; mutex-shared-by-baseproxy2xi-of-BaseProxy4Xi: " + str(BaseProxy4Xi._mutex)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                 
        message2print = "{(Step 5/5f) returning" + "; init-of-BaseProxy4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
    #</methodize4xi>
                                     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "connect-of-BaseProxy4Xi" usage = "connects with (self)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _connect(self):
        '''
        BaseProxy4Xi method _connect: connects with (self)
        '''
        class2print = 'BaseProxy4Xi'
        method2print = '_connect'
        
        message2print = "{started" + "; connect-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
          
        message2print = "{commented; connecttion-commented-by-connect-of-BaseProxy4Xi: " + " BaseProxy4Xi is making connection to a manager" + ";time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------           
        name2xi = get_name_current_process4xi() #dragonized

        message2print = "{set" + "; name2xi-set-by-connect-of-BaseProxy4Xi to: " + str(name2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
                
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        if threading.current_thread().name != 'MainThread':
            name2xi += '|' + threading.current_thread().name
            
            message2print = "{upgraded" + "; name2xi-upgraded-by-connect-of-BaseProxy4Xi: " + str(name2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
        message2print = "{passing" + "; baseproxy2xi-token-address-passing-to-Client-by-connect-of-BaseProxy4Xi: " + str(self._token.address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
            
             
        message2print = "{passing" + "; baseproxy2xi-authkey-passing-to-Client-by-connect-of-BaseProxy4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
              
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        
        connection2xi = self._Client(self._address, BaseProxy4Xi._cf2xi.get_default_family(), self._authkey)

               
        message2print = "{connected" + "; connection2xi-connected-by-connect-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
 
        message2print = "{passing" + "; connection2xi-passing-to-dispatch4xi-by-connect-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passing" + "; None-passing-to-dispatch4xi-by-connect-of-BaseProxy4Xi: " + str(None) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
     
        message2print = "{passing" + "; 'accept_connection-passing-to-dispatch4xi-by-connect-of-BaseProxy4Xi: " + str('accept_connection') + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
     
        message2print = "{passing" + "; name2xi-passing-to-dispatch4xi-by-connect-of-BaseProxy4Xi: " + str(name2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                 
        message2print = "{calling" + "; function-calling: dispatch4xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                    
             
        
        dispatch4xi(connection2xi, None, 'accept_connection', (name2xi,))
        
        message2print = "{dispatched" + "; connection2xi-dispatched-by-connect-of-BaseProxy4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        omega2sleep4xi(class2print, method2print)                    # give time to others
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        self._tls.connection = connection2xi
        
        message2print = "{set" + "; baseproxy2xi-tls-connection-set-by-connect-of-BaseProxy4Xi to: " + str(connection2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        message2print = "{returning" + "; connect-of-BaseProxy4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)            
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
                     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "callmethod-of-BaseProxy4Xi" usage = "tries to call a method of the referrent and returns a copy of the result">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _callmethod(self, methodname, args=(), kwds={}):
        '''
        BaseProxy4Xi method _callmethod: tries to call a method of the referrent and returns a copy of the result
        '''
        class2print = 'BaseProxy4Xi'
        method2print = '_callmethod'
        
        message2print = "{started" + "; callmethod-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        try:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
            connection2xi = self._tls.connection

            message2print = "{connected" + "; connection2xi-connected-by-callmethod-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
  
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                
        except AttributeError:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
            message2print = "{hit; attributeError-hit: thread did not own a connection ;time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            self._connect()
        
            message2print = "{called" + "; baseproxy2xi-connect-called-by-callmethod-of-BaseProxy4Xi: " + str(self._connect) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
      
            
            
            connection2xi = self._tls.connection
            

            message2print = "{connected" + "; connection2xi-connected-by-callmethod-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
  
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 

        connection2xi.send((self._id, methodname, args, kwds))

        message2print = "{sent" + "; baseproxy2xi-id-sent-by-callmethod-of-BaseProxy4Xi: " + str(self._id)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{sent" + "; methodname-sent-by-callmethod-of-BaseProxy4Xi: " + str(methodname)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        

        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        omega2sleep4xi(class2print, method2print)  #give time to others     
        
        
        kind2xi, result2xi = connection2xi.recv()

      
        message2print = "{received" + "; kind2xi-received-by-callmethod-of-BaseProxy4Xi: "  + str(kind2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
              
        message2print = "{received" + "; result2xi-received-by-callmethod-of-BaseProxy4Xi: "  + str(result2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                  
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        if kind2xi == get_return_manager4xi(): #RETURN__MANAGER4XI
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            message2print = "{returning" + "; result2xi-returning-by-callmethod-of-BaseProxy4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)            
            
            return result2xi
        elif kind2xi == get_proxy_manager4xi(): #PROXY_MANAGER4XI
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            exposed2xi, token2xi = result2xi
            
        
            message2print = "{set" + "; exposed2xi-set-by-callmethod-of-BaseProxy4Xi to: " + str(exposed2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
               
            message2print = "{set" + "; token2xi-set-by-callmethod-of-BaseProxy4Xi to: " + str(token2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
      
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
            proxytype2xi = self._manager._registry[token2xi.typeid][-1]
            
            message2print = "{typed" + "; proxytype2xi-typed-by-callmethod-of-BaseProxy4Xi: " + str(proxytype2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
            proxy2xi = proxytype2xi(
                token2xi, self._serializer, manager=self._manager,
                authkey=self._authkey, exposed=exposed2xi
                )

            message2print = "{proxied" + "; proxy2xi-proxied-by-callmethod-of-BaseProxy4Xi: " + str(proxy2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                    
            connection2xi = self._Client(token2xi.address, BaseProxy4Xi._cf2xi.get_default_family(), self._authkey)
            
            message2print = "{connected" + "; connection2xi-connected-by-callmethod-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
                      
            dispatch4xi(connection2xi, None, 'decref', (token2xi.id,))
            
            message2print = "{dispatched" + "; connection2xi-dispatched-by-callmethod-of-BaseProxy4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            message2print = "{returning" + "; proxy2xi-returning-by-callmethod-of-BaseProxy4Xi: " + str(proxy2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
                               
            return proxy2xi
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
        
        # -----------------------------------------------------------------------------------------    
        # Erroneous case
        # -----------------------------------------------------------------------------------------         
        message2print = "{passing" + "; kind2xi-passing-to-convert-to-error4xi-by-callmethod-of-BaseProxy4Xi: " + str(kind2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           

        message2print = "{passing" + "; result2xi-passing-to-convert-to-error4xi-by-callmethod-of-BaseProxy4Xi: " + str(result2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
             
        message2print = "{raising" + "; result2xi-raising-by-callmethod-of-BaseProxy4Xi: "  + str(convert_to_error4x) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
          
        raise convert_to_error4xi(kind2xi, result2xi)
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
            

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getvalue-of-BaseProxy4Xi" usage = "gets a copy of the value of the referent">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _getvalue(self):
        '''
        BaseProxy4Xi method _getvalue: gets a copy of the value of the referent
        '''
        class2print = "BaseProxy4Xi"
        method2print = "_getvalue"
        
        message2print = "{started" + "; getvalue-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
          
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; baseproxy2xi-callmethod-function-returning-by-getvalue-of-BaseProxy4Xi: " + str(self._callmethod) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)     
        
        return self._callmethod(get_getvalue_manager4xi()) #GETVALUE_MANAGER4XI
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "incref-of-BaseProxy4Xi" usage = "increfs with (self)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _incref(self):
        '''
        BaseProxy4Xi method _incref: increfs with (self)
        '''
        class2print = "BaseProxy4Xi"
        method2print = "_incref"
        
        message2print = "{started" + "; incref-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
          
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 

        message2print = "{passing" + "; baseproxy2xi-token-address-passing-to-dispatch4xi-by-incref-of-BaseProxy4Xi: " + str(self._token.address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passing" + "; baseproxy2xi-authkey-passing-to-dispatch4xi-by-incref-of-BaseProxy4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
      
               
        connection2xi = self._Client(self._token.address, BaseProxy4Xi._cf2xi.get_default_family(), self._authkey)  
        
        message2print = "{connected" + "; connection2xi-connected-by-incref-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        
        dispatch4xi(connection2xi, None, 'incref', (self._id,))
        
        message2print = "{dispatched" + "; connection2xi-dispatched-by-incref-of-BaseProxy4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
             
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         

        self._idset.add(self._id)
        
        message2print = "{created" + "; baseproxy2xi-idset-add-created-by-incref-of-BaseProxy4Xi: " + str(self._idset.add) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        state = self._manager and self._manager._state

        message2print = "{stated" + "; state-stated-by-incref-of-BaseProxy4Xi: " + str(state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        
        self._close = Finalize4Xi(
            self, BaseProxy4Xi._decref,
            args=(self._token, self._authkey, state,
                  self._tls, self._idset, self._Client),
            exitpriority=get_exitpriority_manager4xi())

        message2print = "{closed" + "; baseproxy2xi-close-closed-by-incref-of-BaseProxy4Xi: " + str(self._close) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
            
        
        message2print = "{returning" + "; incref-of-BaseProxy4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)         
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
    
        
        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "decref-of-BaseProxy4Xi" usage = "decrefs with (token, authkey, state, tls, idset, _Client)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @staticmethod
    def _decref(token, authkey, state, tls, idset, _Client):
        '''
        BaseProxy4Xi method _decref: decrefs with (token, authkey, state, tls, idset, _Client)
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = "BaseProxy4Xi"
        method2print = "_decref"
        
        
        message2print = "{started" + "; decref-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  

        message2print = "{passed; token-passed-to-decref-of-BaseProxy4Xi: " + str(token) + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed; uthkey-passed-to-decref-of-BaseProxy4Xi: " + str(authkey) + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed; state-passed-to-decref-of-BaseProxy4Xi: " + str(state) + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed; tls-passed-to-decref-of-BaseProxy4Xi: " + str(tls) + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed; idset-passed-to-decref-of-BaseProxy4Xi: " + str(idset) + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed; Client-passed-to-decref-of-BaseProxy4Xi: " + str(_Client) + "}"
        print4xi(class2print, method2print, message2print)           
                
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        idset.discard(token.id)

        message2print = "{discarded" + "; token-id-discarded-by-decref-of-BaseProxy4Xi: "  + str(token.id) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                       
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        # check whether manager is still alive
        if state is None or state.value == State4Xi.STARTED:
            message2print = "{checked" + "; manager-status: " + "alive" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            # tell manager this process no longer cares about referent
            try: 
                message2print = "{telling" + "; status-telling-by-decref-of-BaseProxy4Xi: this process no longer cares about referent"  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                                
                connection2xi = _Client(token.address, BaseProxy4Xi._cf2xi.get_default_family(), authkey)
                
                message2print = "{connected" + "; connection2xi-connected-by-decref-of-BaseProxy4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                          
                
                
                dispatch4xi(connection2xi, None, 'decref', (token.id,))
                
                message2print = "{dispatched" + "; connection2xi-dispatched-by-decref-of-BaseProxy4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
            except Exception as e:
                message2print = "{hit; exception-hit-by-decref-of-BaseProxy4Xi: BaseProxy4Xi method _decref failed; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                

        else:
            message2print = "{checked; manager-status-checked-by-decref-of-BaseProxy4Xi: already shutdown" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
     
          
        
            
        # -----------------------------------------------------------------------------------------    
        # check whether we can close this thread's connection because
        # the process owns no more references to objects for this manager
        #
        # ----------------------------------------------------------------------------------------- 
        
        if not idset and hasattr(tls, 'connection'):
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            message2print = "{concluded" + "; references-concluded-by-decref-of-BaseProxy4Xi: process owns no more references to objects for the  manager" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            
            message2print = "{concluded" + "; thread-concluded-by-decref-of-BaseProxy4Xi: thread has no more proxies so closing connection time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{decided" + "; closing-decided-by-decref-of-BaseProxy4Xi: thread connection" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  

            
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
            tls.connection.close()
            
            message2print = "{closed" + "; connection-closed-by-decref-of-BaseProxy4Xi: " + str(tls.connection.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            message2print = "{deleting" + "; connection-deleting-by-decref-of-BaseProxy4Xi: "  + str(tls.connection) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                        
            del tls.connection
            
            
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                 
        
        message2print = "{returning" + "; decref-of-BaseProxy4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                         
            
            
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
        
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "after-fork-of-BaseProxy4Xi" usage = "increfs with (self)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _after_fork(self):
        '''
        BaseProxy4Xi method _after_fork: increfs with (self) 
        '''
        class2print = "BaseProxy4Xi"
        method2print = "_after_fork"
        
        
        message2print = "{started" + "; after-fork-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
         
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        self._manager = None
        message2print = "{set" + "; baseproxy2xi-manager-set-by-after-fork-of-BaseProxy4Xi to: " + str(self._manager)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        try:
            self._incref()
            
            message2print = "{called" + "; baseproxy2xi-incref-called-by-after-fork-of-BaseProxy4Xi: " + str(self._incref) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            
        except Exception as e:
            message2print = "{hit; exception-hit-by-after-fork-of-BaseProxy4Xi: incref failed because the proxy may just be for a manager which has shutdown; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            # incref failed 
        
        message2print = "{returning" + "; after-fork-of-BaseProxy4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)          
                
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reduce-of-BaseProxy4Xi" usage = "reduces and returns with (self)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __reduce__(self):
        '''
        BaseProxy4Xi method __reduce__: reduces and returns with (self) 
        '''
        class2print = "BaseProxy4Xi"
        method2print = "__reduce__"
        
        
        message2print = "{started" + "; reduce-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
         
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        kwds = {}
        
        message2print = "{checking" + "; spawning-checking-by-reduce-of-BaseProxy4Xi: "  + str(Popen4Xi.thread_is_spawning)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        if Popen4Xi.thread_is_spawning():
            kwds['authkey'] = self._authkey
            
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        if getattr(self, '_isauto', False):
            kwds['exposed'] = self._exposed_
            
            message2print = "{passing" + "; baseproxy2xi-token-passing-to-return-by-reduce-of-BaseProxy4Xi: " + str(self._token) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
   
            message2print = "{passing" + "; baseproxy2xi-serializer-passing-to-return-by-reduce-of-BaseProxy4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            
            message2print = "{returning" + "; RebuildProxy4Xi-returning-by-reduce-of-BaseProxy4Xi: " + str(RebuildProxy4Xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{returning" + "; AutoProxy4Xi-returning-by-reduce-of-BaseProxy4Xi: " + str(AutoProxy4Xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
              
            return (RebuildProxy4Xi,
                    (AutoProxy4Xi, self._token, self._serializer, kwds))
        else:
            message2print = "{returning" + "; RebuildProxy4Xi-class-returning-by-reduce-of-BaseProxy4Xi: " + str(RebuildProxy4Xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
        
            return (RebuildProxy4Xi,
                    (type(self), self._token, self._serializer, kwds))
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
    #</methodize4xi>
            
        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "deepcopy-of-BaseProxy4Xi" usage = "copies and returns with (self, memo)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __deepcopy__(self, memo):
        '''
        BaseProxy4Xi method __deepcopy__: copies and returns with (self, memo)
        '''
        class2print = "BaseProxy4Xi"
        method2print = "__deepcopy__"
        
        
        message2print = "{started" + "; deepcopy-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        message2print = "{returning" + "; baseproxy2xi-getvalue-function-returning-by-deepcopy-of-BaseProxy4Xi: " + str(self._getvalue) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        
        return self._getvalue()
    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "repr-of-BaseProxy4X" usage = "presents and returns with (self)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __repr__(self):
        '''
        BaseProxy4Xi method __repr__: presents and returns with (self) 
        '''
        class2print = "BaseProxy4Xi"
        method2print = "__repr__"
        
        
        message2print = "{started" + "; repr-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; token-type-returning-by-repr-of-BaseProxy4Xi: token-type, see trace"  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        
        return '<%s object, typeid %r at %s>' % \
               (type(self).__name__, self._token.typeid, '0x%x' % id(self))
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "str-of-BaseProxy4Xi" usage = "returns representation of the referent (or a fall-back if that fails)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __str__(self):
        '''
        BaseProxy4Xi method __str___: returns representation of the referent (or a fall-back if that fails)
        '''
        class2print = "BaseProxy4Xi"
        method2print = "__str__"
        
        
        message2print = "{started" + "; str-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        try:
            message2print = "{returning" + "; baseproxy2xi-callmethod-function-returning-by-str-of-BaseProxy4Xi: " + str(self._callmethod) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            
            return self._callmethod('__repr__')
        except Exception:
            message2print = "{hit" + "; Exception-hit-by-str-of-BaseProxy4Xi: "  + str(self._callmethod) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            return repr(self)[:-1] + "; '__str__()' failed>"
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-BaseProxy4Xi" usage = "senses class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses BaseProxy4Xi  class objects
        '''
  
        
        class2print = 'BaseProxy4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  

        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{sensing" + "; BaseProxy4Xi-sensing-by-sense-of-BaseProxy4Xi: " + str(BaseProxy4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        kb2xi = sense_object4xi(self)
        
        try:
            kb2xi['address-to-local-of-baseproxy2xi'] = str(BaseProxy4Xi._address_to_local)
            kb2xi['mutex-of-baseproxy2xi'] = str(BaseProxy4Xi._mutex)            
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-BaseProxy4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
                
       
        message2print = "{sensed" + "; BaseProxy4Xi-sensed-by-sense-of-BaseProxy4Xi: " + str(BaseProxy4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-BaseProxy4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                 
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-BaseProxy4X" usage = "reasons class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons BaseProxy4Xi class objects
        '''
       
        
        class2print = 'BaseProxy4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-BaseProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; BaseProxy4Xi-reasoning-by-reason-of-BaseProxy4Xi: "  + str(BaseProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        
        kb4xi = reason_object4xi(self)
        
        
        
        message2print = "{reasoned" + "; BaseProxy4Xi-reasoned-by-reason-of-BaseProxy4Xi: "  + str(BaseProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-BaseProxy4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                 
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-BaseProxy4Xi" usage = "aces class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces BaseProxy4Xi class objects 
        '''
   
        
        
        class2print = 'BaseProxy4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-BaseProxy4Xi-started-by-ace-of-BaseProxy4Xi: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; BaseProxy4Xi-acing-by-ace-of-BaseProxy4Xi: " + "BaseProxy4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
      
        kb8xi = ace_object4xi(self)  
        
        message2print = "{aced" + "; BaseProxy4Xi-aced-by-ace-of-BaseProxy4Xi: " + "BaseProxy4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-BaseProxy4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)       
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                    
    #</methodize4xi>
#</class4xi>
#</program4xi> 

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of baseproxy4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>


